package cn.ibizlab.assetmanagement.core.apply.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import io.seata.spring.annotation.GlobalTransactional;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.apply.domain.EAMApply;
import cn.ibizlab.assetmanagement.core.apply.filter.EAMApplySearchContext;
import cn.ibizlab.assetmanagement.core.apply.service.IEAMApplyService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.apply.mapper.EAMApplyMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[维护申请] 服务对象接口实现
 */
@Slf4j
@Service("EAMApplyServiceImpl")
public class EAMApplyServiceImpl extends ServiceImpl<EAMApplyMapper, EAMApply> implements IEAMApplyService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.apply.service.IEAMApplyTypeService eamapplytypeService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetService eamassetService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.location.service.IEAMLocationService eamlocationService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMApply et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamapplyid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMApply> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMApply et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamapplyid",et.getEamapplyid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamapplyid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMApply> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMApply get(String key) {
        EAMApply et = getById(key);
        if(et==null){
            et=new EAMApply();
            et.setEamapplyid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMApply getDraft(EAMApply et) {
        return et;
    }

    @Override
    public boolean checkKey(EAMApply et) {
        return (!ObjectUtils.isEmpty(et.getEamapplyid()))&&(!Objects.isNull(this.getById(et.getEamapplyid())));
    }
    @Override
    @Transactional
    public boolean save(EAMApply et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMApply et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMApply> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMApply> list) {
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMApply> selectByEamapplytypeid(String eamapplytypeid) {
        return baseMapper.selectByEamapplytypeid(eamapplytypeid);
    }

    @Override
    public void removeByEamapplytypeid(String eamapplytypeid) {
        this.remove(new QueryWrapper<EAMApply>().eq("eamapplytypeid",eamapplytypeid));
    }

	@Override
    public List<EAMApply> selectByAssetid(String assetid) {
        return baseMapper.selectByAssetid(assetid);
    }

    @Override
    public void removeByAssetid(String assetid) {
        this.remove(new QueryWrapper<EAMApply>().eq("assetid",assetid));
    }

	@Override
    public List<EAMApply> selectByEamlocationid(String eamlocationid) {
        return baseMapper.selectByEamlocationid(eamlocationid);
    }

    @Override
    public void removeByEamlocationid(String eamlocationid) {
        this.remove(new QueryWrapper<EAMApply>().eq("eamlocationid",eamlocationid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMApply> searchDefault(EAMApplySearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMApply> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMApply>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMApply> getEamapplyByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMApply> getEamapplyByEntities(List<EAMApply> entities) {
        List ids =new ArrayList();
        for(EAMApply entity : entities){
            Serializable id=entity.getEamapplyid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



