package cn.ibizlab.assetmanagement.core.asset.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import io.seata.spring.annotation.GlobalTransactional;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.asset.domain.EAMAsset;
import cn.ibizlab.assetmanagement.core.asset.filter.EAMAssetSearchContext;
import cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.asset.mapper.EAMAssetMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[资产] 服务对象接口实现
 */
@Slf4j
@Service("EAMAssetServiceImpl")
public class EAMAssetServiceImpl extends ServiceImpl<EAMAssetMapper, EAMAsset> implements IEAMAssetService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.apply.service.IEAMApplyService eamapplyService;

    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetService eamassetService = this;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.plan.service.IEAMPlanLineService eamplanlineService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.plan.service.IEAMPlanService eamplanService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.workorder.service.IEAMWorkOrderService eamworkorderService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetTypeService eamassettypeService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.location.service.IEAMLocationService eamlocationService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMAsset et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getAssetid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMAsset> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMAsset et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("assetid",et.getAssetid())))
            return false;
        CachedBeanCopier.copy(get(et.getAssetid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMAsset> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMAsset get(String key) {
        EAMAsset et = getById(key);
        if(et==null){
            et=new EAMAsset();
            et.setAssetid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMAsset getDraft(EAMAsset et) {
        return et;
    }

    @Override
    public boolean checkKey(EAMAsset et) {
        return (!ObjectUtils.isEmpty(et.getAssetid()))&&(!Objects.isNull(this.getById(et.getAssetid())));
    }
    @Override
    @Transactional
    public boolean save(EAMAsset et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMAsset et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMAsset> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMAsset> list) {
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMAsset> selectByEamassettypeid(String eamassettypeid) {
        return baseMapper.selectByEamassettypeid(eamassettypeid);
    }

    @Override
    public void removeByEamassettypeid(String eamassettypeid) {
        this.remove(new QueryWrapper<EAMAsset>().eq("eamassettypeid",eamassettypeid));
    }

	@Override
    public List<EAMAsset> selectByPassetid(String assetid) {
        return baseMapper.selectByPassetid(assetid);
    }

    @Override
    public void removeByPassetid(String assetid) {
        this.remove(new QueryWrapper<EAMAsset>().eq("passetid",assetid));
    }

	@Override
    public List<EAMAsset> selectByEamlocationid(String eamlocationid) {
        return baseMapper.selectByEamlocationid(eamlocationid);
    }

    @Override
    public void removeByEamlocationid(String eamlocationid) {
        this.remove(new QueryWrapper<EAMAsset>().eq("eamlocationid",eamlocationid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMAsset> searchDefault(EAMAssetSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMAsset> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMAsset>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMAsset> getEamassetByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMAsset> getEamassetByEntities(List<EAMAsset> entities) {
        List ids =new ArrayList();
        for(EAMAsset entity : entities){
            Serializable id=entity.getAssetid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



