package cn.ibizlab.assetmanagement.core.plan.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import io.seata.spring.annotation.GlobalTransactional;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.plan.domain.EAMPlan;
import cn.ibizlab.assetmanagement.core.plan.filter.EAMPlanSearchContext;
import cn.ibizlab.assetmanagement.core.plan.service.IEAMPlanService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.plan.mapper.EAMPlanMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[维护计划] 服务对象接口实现
 */
@Slf4j
@Service("EAMPlanServiceImpl")
public class EAMPlanServiceImpl extends ServiceImpl<EAMPlanMapper, EAMPlan> implements IEAMPlanService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.plan.service.IEAMPlanCDTService eamplancdtService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.plan.service.IEAMPlanLineService eamplanlineService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.plan.service.IEAMPlanScheduleService eamplanscheduleService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetService eamassetService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.location.service.IEAMLocationService eamlocationService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMPlan et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamplanid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMPlan> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMPlan et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamplanid",et.getEamplanid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamplanid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMPlan> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMPlan get(String key) {
        EAMPlan et = getById(key);
        if(et==null){
            et=new EAMPlan();
            et.setEamplanid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMPlan getDraft(EAMPlan et) {
        return et;
    }

    @Override
    public boolean checkKey(EAMPlan et) {
        return (!ObjectUtils.isEmpty(et.getEamplanid()))&&(!Objects.isNull(this.getById(et.getEamplanid())));
    }
    @Override
    @Transactional
    public boolean save(EAMPlan et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMPlan et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMPlan> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMPlan> list) {
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMPlan> selectByAssetid(String assetid) {
        return baseMapper.selectByAssetid(assetid);
    }

    @Override
    public void removeByAssetid(String assetid) {
        this.remove(new QueryWrapper<EAMPlan>().eq("assetid",assetid));
    }

	@Override
    public List<EAMPlan> selectByEamlocationid(String eamlocationid) {
        return baseMapper.selectByEamlocationid(eamlocationid);
    }

    @Override
    public void removeByEamlocationid(String eamlocationid) {
        this.remove(new QueryWrapper<EAMPlan>().eq("eamlocationid",eamlocationid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMPlan> searchDefault(EAMPlanSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMPlan> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMPlan>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMPlan> getEamplanByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMPlan> getEamplanByEntities(List<EAMPlan> entities) {
        List ids =new ArrayList();
        for(EAMPlan entity : entities){
            Serializable id=entity.getEamplanid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



