package cn.ibizlab.assetmanagement.core.workorder.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import io.seata.spring.annotation.GlobalTransactional;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.workorder.domain.EAMWorkOrder;
import cn.ibizlab.assetmanagement.core.workorder.filter.EAMWorkOrderSearchContext;
import cn.ibizlab.assetmanagement.core.workorder.service.IEAMWorkOrderService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.workorder.mapper.EAMWorkOrderMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[工单] 服务对象接口实现
 */
@Slf4j
@Service("EAMWorkOrderServiceImpl")
public class EAMWorkOrderServiceImpl extends ServiceImpl<EAMWorkOrderMapper, EAMWorkOrder> implements IEAMWorkOrderService {


    protected cn.ibizlab.assetmanagement.core.workorder.service.IEAMWorkOrderService eamworkorderService = this;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetService eamassetService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.location.service.IEAMLocationService eamlocationService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.workorder.service.IEAMWOTypeService eamwotypeService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMWorkOrder et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getWorkorderid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMWorkOrder> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMWorkOrder et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("workorderid",et.getWorkorderid())))
            return false;
        CachedBeanCopier.copy(get(et.getWorkorderid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMWorkOrder> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMWorkOrder get(String key) {
        EAMWorkOrder et = getById(key);
        if(et==null){
            et=new EAMWorkOrder();
            et.setWorkorderid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMWorkOrder getDraft(EAMWorkOrder et) {
        return et;
    }

    @Override
    public boolean checkKey(EAMWorkOrder et) {
        return (!ObjectUtils.isEmpty(et.getWorkorderid()))&&(!Objects.isNull(this.getById(et.getWorkorderid())));
    }
    @Override
    @Transactional
    public boolean save(EAMWorkOrder et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMWorkOrder et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMWorkOrder> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMWorkOrder> list) {
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMWorkOrder> selectByAssetid(String assetid) {
        return baseMapper.selectByAssetid(assetid);
    }

    @Override
    public void removeByAssetid(String assetid) {
        this.remove(new QueryWrapper<EAMWorkOrder>().eq("assetid",assetid));
    }

	@Override
    public List<EAMWorkOrder> selectByEamlocationid(String eamlocationid) {
        return baseMapper.selectByEamlocationid(eamlocationid);
    }

    @Override
    public void removeByEamlocationid(String eamlocationid) {
        this.remove(new QueryWrapper<EAMWorkOrder>().eq("eamlocationid",eamlocationid));
    }

	@Override
    public List<EAMWorkOrder> selectByPworkorderid(String workorderid) {
        return baseMapper.selectByPworkorderid(workorderid);
    }

    @Override
    public void removeByPworkorderid(String workorderid) {
        this.remove(new QueryWrapper<EAMWorkOrder>().eq("pworkorderid",workorderid));
    }

	@Override
    public List<EAMWorkOrder> selectByEamwotypeid(String eamwotypeid) {
        return baseMapper.selectByEamwotypeid(eamwotypeid);
    }

    @Override
    public void removeByEamwotypeid(String eamwotypeid) {
        this.remove(new QueryWrapper<EAMWorkOrder>().eq("eamwotypeid",eamwotypeid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMWorkOrder> searchDefault(EAMWorkOrderSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMWorkOrder> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMWorkOrder>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMWorkOrder> getEamworkorderByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMWorkOrder> getEamworkorderByEntities(List<EAMWorkOrder> entities) {
        List ids =new ArrayList();
        for(EAMWorkOrder entity : entities){
            Serializable id=entity.getWorkorderid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



